/* Part of Misnamed Filter Configurator (see Program.cs)
 * by Alex Vallat <www.byalexv.co.uk>
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;

namespace MisnamedFilter.Configurator
{
	public partial class Main : Form
	{
		private static readonly Font sDefaultFont = new Font("MS Shell Dlg 2", 8);

		public Main()
		{
			SetFont(sDefaultFont);

			InitializeComponent();

			if (!File.Exists(GetHelpFilePath()))
				this.HelpButton = false;
		}

		#region Save and Load Settings
		private Settings mSettings;
		protected override void OnShown(EventArgs e)
		{
			base.OnShown(e);
			LoadSettings();
		}

		private void mOK_Click(object sender, EventArgs e)
		{
			SaveSettings();
			this.Close();
		}

		private void mCancel_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void SaveSettings()
		{
			mSettings.AutoLearn = mAutoLearn.Checked;
			mSettings.AddressSets.Clear();
			foreach (AddressSet addressSet in mAddressSets.Items)
			{
				mSettings.AddressSets.Add(addressSet);
			}

			mSettings.Save();
		}
		
		private void LoadSettings()
		{
			if(mSettings == null)
			{
				mSettings = new Settings();
			}
			mSettings.Reload();

			mAutoLearn.Checked = mSettings.AutoLearn;

			mAddressSets.Items.Clear();
			foreach (AddressSet addressSet in mSettings.AddressSets)
			{
				mAddressSets.Items.Add(addressSet);
			}

			if (mAddressSets.Items.Count == 0)
			{
				//Create a default item to get started
				AddNewAddressSet();
			}
			else
			{
				mAddressSets.SelectedIndex = 0;
			}

			PopulateControlsFromAddressSet();
		}
		#endregion

		#region Font
		public override Font Font
		{
			get
			{
				return base.Font;
			}
			set
			{
				//Disallow direct font setting by the Windows Form Designer.  Call SetFont instead, if absolutely necessary.
			}
		}

		public void SetFont(Font font)
		{
			base.Font = font;
		}
		#endregion

		#region Selection Change UI
		private bool mSuspendSelectionTracking = false;
		private void mAddressSets_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!mSuspendSelectionTracking)
			{
				PopulateControlsFromAddressSet();
			}
		}

		private void PopulateControlsFromAddressSet()
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			mEmailAddressPatterns.Items.Clear();
			mEmailAddressPattern.Clear();
			mNamePatterns.Items.Clear();
			mNamePattern.Clear();

			if (addressSet == null)
			{
				mRemoveAddressSet.Enabled = false;
				mAddressSetBox.Enabled = false;
				mAddressSetBox.Text = "No address set selected";
				mDescriptiveName.Clear();
			}
			else
			{
				mRemoveAddressSet.Enabled = true;
				mAddressSetBox.Enabled = true;
				mAddressSetBox.Text = addressSet.DescriptiveName;
				mDescriptiveName.Text = addressSet.DescriptiveName;

				foreach (string addressPattern in addressSet.AddressPatterns)
				{
					mEmailAddressPatterns.Items.Add(addressPattern);
				}
				if (addressSet.AddressPatterns.Count > 0) mEmailAddressPatterns.SelectedIndex = 0;
				foreach (string namePattern in addressSet.NamePatterns)
				{
					mNamePatterns.Items.Add(namePattern);
				}
				if (addressSet.NamePatterns.Count > 0) mNamePatterns.SelectedIndex = 0;
			}

			PopulateControlsFromEmailAddressPattern();
			PopulateControlsFromNamePattern();
		}

		private void mEmailAddressPatterns_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!mSuspendSelectionTracking)
			{
				PopulateControlsFromEmailAddressPattern();
			}
		}
		private void PopulateControlsFromEmailAddressPattern()
		{
			string emailAddressPattern = mEmailAddressPatterns.SelectedItem as string;
			if (emailAddressPattern == null)
			{
				mRemoveEmailAddressPattern.Enabled = false;
				mEmailAddressPattern.Clear();
				mEmailAddressPattern.Enabled = false;
			}
			else
			{
				mRemoveEmailAddressPattern.Enabled = true;
				mEmailAddressPattern.Enabled = true;
				mEmailAddressPattern.Text = emailAddressPattern;
			}
		}

		private void mNamePatterns_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (!mSuspendSelectionTracking)
			{
				PopulateControlsFromNamePattern();
			}
		}
		private void PopulateControlsFromNamePattern()
		{
			string namePattern = mNamePatterns.SelectedItem as string;
			if (namePattern == null)
			{
				mRemoveNamePattern.Enabled = false;
				mNamePattern.Clear();
				mNamePattern.Enabled = false;
			}
			else
			{
				mRemoveNamePattern.Enabled = true;
				mNamePattern.Enabled = true;
				mNamePattern.Text = namePattern;
			}
		}
		#endregion

		#region Binding
		private void mDescriptiveName_TextChanged(object sender, EventArgs e)
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			if (addressSet != null)
			{
				if (addressSet.DescriptiveName != mDescriptiveName.Text)
				{
					addressSet.DescriptiveName = mDescriptiveName.Text;
					mAddressSetBox.Text = addressSet.DescriptiveName;
					mSuspendSelectionTracking = true;
					mAddressSets.Items[mAddressSets.Items.IndexOf(addressSet)] = addressSet;
					mSuspendSelectionTracking = false;
				}
			}
		}
		private void mEmailAddressPattern_TextChanged(object sender, EventArgs e)
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			if (addressSet != null)
			{
				int selectedEmailAddressPattern = mEmailAddressPatterns.SelectedIndex;
				if (selectedEmailAddressPattern > -1)
				{
					string emailAddressPattern = mEmailAddressPattern.Text;
					if (addressSet.AddressPatterns[selectedEmailAddressPattern] != emailAddressPattern)
					{
						addressSet.AddressPatterns[selectedEmailAddressPattern] = emailAddressPattern;
						mSuspendSelectionTracking = true;
						mEmailAddressPatterns.Items[selectedEmailAddressPattern] = emailAddressPattern;
						mSuspendSelectionTracking = false;
						UpdateTesters();
					}
				}
			}
		}
		private void mNamePattern_TextChanged(object sender, EventArgs e)
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			if (addressSet != null)
			{
				int selectedNamePattern = mNamePatterns.SelectedIndex;
				if (selectedNamePattern > -1)
				{
					string namePattern = mNamePattern.Text;
					if (addressSet.NamePatterns[selectedNamePattern] != namePattern)
					{
						addressSet.NamePatterns[selectedNamePattern] = namePattern;
						mSuspendSelectionTracking = true;
						mNamePatterns.Items[selectedNamePattern] = namePattern;
						mSuspendSelectionTracking = false;
						UpdateTesters();
					}
				}
			}
		}
		#endregion

		#region Adding
		private void mSimpleAdd_Click(object sender, EventArgs e)
		{
			SimpleAdd();
		}
		private void SimpleAdd()
		{
			SimpleAdd simpleAdd = new SimpleAdd(this);
			if (simpleAdd.ShowDialog(this) == DialogResult.OK)
			{
				//Try and find an address set which already contains the specified address pattern
				string addressPattern = simpleAdd.AddressPattern;
				string namePattern = simpleAdd.NamePattern;

				AddressSet addressSet = null;
				foreach (AddressSet potentialAddressSet in GetAddressSets())
				{
					if (potentialAddressSet.AddressPatterns.Contains(addressPattern))
					{
						//This set contains the exact address pattern already, so just add the name pattern to it
						addressSet = potentialAddressSet;
						break;
					}
				}
				if (addressSet == null)
				{
					//No matching address set found, so create a new one.
					addressSet = new AddressSet();
					addressSet.DescriptiveName = simpleAdd.AddressSetName;
					addressSet.AddressPatterns.Add(addressPattern);
					mAddressSets.Items.Add(addressSet);
				}
				if (!addressSet.NamePatterns.Contains(namePattern))
				{
					//Add the name pattern
					addressSet.NamePatterns.Add(namePattern);
				}

				//Keep everything in synch.
				mAddressSets.SelectedItem = addressSet;
				PopulateControlsFromAddressSet();
				UpdateTesters();
			}
		}

		private void mAddAddressSet_Click(object sender, EventArgs e)
		{
			AddNewAddressSet();
		}
		private void AddNewAddressSet()
		{
			AddressSet addressSet = new AddressSet();
			addressSet.DescriptiveName = "<new address set>";
			addressSet.AddressPatterns.Add(String.Empty);
			addressSet.NamePatterns.Add(String.Empty);
			mAddressSets.SelectedIndex = mAddressSets.Items.Add(addressSet);
			mDescriptiveName.Focus();
			mDescriptiveName.SelectAll();
			UpdateTesters();
		}

		private void mAddEmailAddressPattern_Click(object sender, EventArgs e)
		{
			AddNewEmailAddressPattern();
		}
		private void AddNewEmailAddressPattern()
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			if (addressSet != null)
			{
				string emailAddressPattern = "<new email address pattern>";
				addressSet.AddressPatterns.Add(emailAddressPattern);
				mEmailAddressPatterns.SelectedIndex = mEmailAddressPatterns.Items.Add(emailAddressPattern);
				mEmailAddressPattern.Focus();
				mEmailAddressPattern.SelectAll();
				UpdateTesters();
			}
		}

		private void mAddNamePattern_Click(object sender, EventArgs e)
		{
			AddNewNamePattern();
		}
		private void AddNewNamePattern()
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			if (addressSet != null)
			{
				string namePattern = "<new name pattern>";
				addressSet.NamePatterns.Add(namePattern);
				mNamePatterns.SelectedIndex = mNamePatterns.Items.Add(namePattern);
				mNamePattern.Focus();
				mNamePattern.SelectAll();
				UpdateTesters();
			}
		}
		#endregion

		#region Removing
		private void mRemoveAddressSet_Click(object sender, EventArgs e)
		{
			RemoveSelectedAddressSet();
		}
		private void RemoveSelectedAddressSet()
		{
			RemoveSelectedItem(mAddressSets);
		}

		private void mRemoveEmailAddressPattern_Click(object sender, EventArgs e)
		{
			RemoveSelectedEmailAddressPattern();
		}
		private void RemoveSelectedEmailAddressPattern()
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			if (addressSet != null)
			{
				int index = RemoveSelectedItem(mEmailAddressPatterns);
				if (index > -1)
					addressSet.AddressPatterns.RemoveAt(index);
			}
		}

		private void mRemoveNamePattern_Click(object sender, EventArgs e)
		{
			RemoveSelectedNamePattern();
		}
		private void RemoveSelectedNamePattern()
		{
			AddressSet addressSet = mAddressSets.SelectedItem as AddressSet;
			if (addressSet != null)
			{
				int index = RemoveSelectedItem(mNamePatterns);
				if (index > -1)
					addressSet.NamePatterns.RemoveAt(index);
			}
		}

		private int RemoveSelectedItem(ListBox listBox)
		{
			int index = listBox.SelectedIndex;
			if (index > -1)
			{
				listBox.Items.RemoveAt(index);
				listBox.SelectedIndex = Math.Min(listBox.Items.Count - 1, index);
				UpdateTesters();
				return index;
			}
			return -1;
		}
		#endregion

		#region Testers
		private List<Tester> mTesters = new List<Tester>();

		private void UpdateTesters()
		{
			foreach (Tester tester in mTesters)
			{
				tester.BeginTest();
			}
		}
		
		private void mTest_Click(object sender, EventArgs e)
		{
			Tester tester = new Tester(this);
			mTesters.Add(tester);
			tester.Show(this);
		}

		internal IEnumerable<AddressSet> GetAddressSets()
		{
			foreach (AddressSet addressSet in mAddressSets.Items)
			{
				yield return addressSet;
			}
		}
		#endregion

		#region Help
		protected override void OnHelpButtonClicked(CancelEventArgs e)
		{
			ShowHelp();
			e.Cancel = true;
		}

		internal static void ShowHelp()
		{
			//TODO: More advanced help, including context
			string helpFile = GetHelpFilePath();
			if (File.Exists(helpFile))
			{
				System.Diagnostics.Process.Start(helpFile);
			}
		}

		private static string GetHelpFilePath()
		{
			return Path.Combine(Path.GetDirectoryName(typeof(Main).Assembly.Location), "readme.html");
		}
		#endregion
	}
}